############################################################
# タイトル：かんたんメール送信(添付ファイル対応)
# 動作環境：UNIX系OS
# 作者：ORBIT
############################################################
package Sendmail; # クラスのパッケージ名を宣言

use Encode;
use MIME::Base64;

sub new {
	# 引数を受ける
	my ( $class, @args ) = @_;
	my %args = ref $args[0] eq 'HASH' ? %{ $args[0] } : @args;
	my $self = { %args }; #クラスプロパティ
	#++++++++++++++++++++#
	# デフォルト値を指定 #
	#++++++++++++++++++++#
	# Sendmailまでのパス
	$self->{ Pash }     = '/usr/sbin/sendmail'  unless defined $self->{ Pash };
	# 件名
	$self->{ Subject }  = 'NO Subject'          unless defined $self->{ Subject };
	# From
	$self->{ From }     = 'user@localhost'      unless defined $self->{ From };
	# To
	$self->{ To }       = 'user@localhost'      unless defined $self->{ To };
	# 本文
	$self->{ Message }  = 'Message'             unless defined $self->{ Message };
	# 境界
	$self->{ Boundary } = '3xic8QYVM1kQ16YySsF' unless defined $self->{ Boundary };
	# 添付ファイルまでのパス
	$self->{ Files }    = 'null'                unless defined $self->{ Files };
	
	return bless $self , $class;
};

sub sendmail{
	my $self = shift; #クラスプロパティ
	$self->{ To }      = $_[0] if( $_[0] ); # アクセッサを定義
	$self->{ Message } = $_[1] if( $_[1] ); # アクセッサを定義
	$self->{ Files }   = $_[2] if( $_[2] ); # アクセッサを定義
	
	$subject = Encode::encode('MIME-Header-ISO_2022_JP', $self->{ Subject });
	my $message = Encode::decode('utf8', $self->{ Message });
	$message =~ tr/[\x{ff5e}\x{2225}\x{ff0d}\x{ffe0}\x{ffe1}\x{ffe2}]/[\x{301c}\x{2016}\x{2212}\x{00a2}\x{00a3}\x{00ac}]/;
	$message = Encode::encode('iso-2022-jp', $message);
	
	open(SDML,"| $self->{ Pash } -t -i") || die "can't open Sendmail !";
	print SDML "From: " . $self->{ From } . "\n";
	print SDML "To: " . $self->{ To } . "\n";
	print SDML "Subject: " . $subject . "\n";
	print SDML "MIME-Version: 1.0\n";

	my $withfiles;
	if($self->{ Files } eq "null"){
		print SDML "Content-type: text/plain;charset=ISO-2022-JP\n";
		print SDML "Content-Transfer-Encoding: 7bit\n";
		print SDML $message . "\n";
	}else{
		my @workfiles = split( /\,/, $self->{ Files } );
		for( my $count = 0; $count < $#workfiles + 1 ; $count++ ){
			open(IN,"<",$workfiles[$count]);
			my $filesize = -s $workfiles[$count];
			my $filebody;
			my $readsize = read(IN, $filebody, $filesize);
			close(IN);

			my $base64filebody = MIME::Base64::encode($filebody);
			$workfiles[$count] =~ /^.*\/(.+)$/;
			$filename = $1;
			my $mimefilename = Encode::encode('MIME-Header-ISO_2022_JP', $filename);
			my $codefilename = $filename;
			$codefilename =~ s/(\W)/sprintf("%%%02X", ord($1))/ego;

			$withfiles .= "--" . $self->{ Boundary } . "\n" .
			"Content-Type: application/octet-stream;\n" .
			" name=" . $mimefilename . " \n" .
			"Content-Transfer-Encoding: base64\n" .
			"Content-Disposition: attachment;" .
			" filename*=iso-2022-jp'ja'" . $codefilename . " \n\n" .
			$base64filebody . "\n";
		}
		print SDML "Content-Type: Multipart/mixed; boundary=" . $self->{ Boundary } . "\n\n";
		print SDML "--" . $self->{ Boundary } . "\n";
		print SDML "Content-type: text/plain;charset=ISO-2022-JP\n";
		print SDML "Content-Transfer-Encoding: 7bit\n\n";
		print SDML $message . "\n";
		print SDML $withfiles . "\n";
		print SDML "--" . $self->{ Boundary } . "--\n";
	}
	
	close(SDML);
	return 1;
};

1;

__END__

http://www.perl-labo.org/sendmail/filetemp/
http://webmarutaka.blog24.fc2.com/blog-entry-513.html
http://ash.jp/perl/tips_perl01.htm
http://www.emaillab.org/essay/japanese-filename.html

==== ビックリするくらい簡単な利用法 ====

#!/usr/bin/perl

use Sendmail;

# オブジェクトの生成
my $send_obj = Sendmail->new({
        Subject => '件名',                     # 件名
        From    => 'user@mail.goge.net',       # 送信元
        });

# メール送信
$send_obj->sendmail(
                'user@mail.goge.net',          # 送信先
                '添付ファイル無し',            # メッセージ
                );

# メール送信
$send_obj->sendmail(
                'user@mail.goge.net',          # 送信先
                '添付ファイル有り',            # メッセージ
                'file.txt,日本語ファイル.txt'  # 添付ファイル(,で区切ると複数送信可能)
                );